% Code for

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Appendix I of
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Semih Uslu: Pricing and Liquidity in Decentralized Asset Markets
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clc;
clear all;


%__________________________________________________________________________
%
% Parameters
%__________________________________________________________________________

M                        = 180;        % number of search efficiency types
N                        = 90;         % number of risk aversion types

r                        = 0.05;       % discount rate
sigmaD                   = sqrt(2000); % asset payoff volatility

qtl1                     = linspace(0, 1, M)'; % search type generator grid
qtl2                     = linspace(0, 1, N)'; % risk-aversion genera. grid

lambda                   = [icdf('unif',qtl1,5,10)]; % vector of lambdas
pila                     = 1/M*ones(1,M);            % pmf vector of lambda
caplambda                = pila*lambda;              % agg. search speed

gamma                    = [icdf('unif',qtl2,2,5)];  % vector of gammas
piga                     = 1/N*ones(1,N);            % pmf vector of gamma


%__________________________________________________________________________
%
% Matrices to be updated during the iteration
%__________________________________________________________________________

rt                       = zeros(M,N,1); % finalized matrix of r-tilde
rt0                      = zeros(M,N,1); % pre-iteration matrix of r-tilde
rt1                      = zeros(M,N,1); % post-iterarion matrix of r-tilde


%__________________________________________________________________________
%
% Iteration
%__________________________________________________________________________

iter_rt                  = 0;             % iteration counter

test_rt                  = 10;            
% test variable to gauge whether the convergence has been obtained 

tolerance_rt             = 10^(-9);       
% I consider that convergence obtains if test_rt is less than tolerance_rt

for n = 1:N
    
   rt0(:,n,1)            = r+(lambda)./2; % initialize matrix of r-tilde
   % ( rt0(i,n,1) is r-tilde that would obtain in a fully homogeneous model 
   % in which investors have the same search speed of lambda(i) )
end

% use rt0 on the RHS of Equation (I.1) to obtain rt1
while abs(test_rt) > tolerance_rt
    
    for n = 1:N
        
      for m = 1:M
    
        rt1(m,n,1)       = r;
    
        for k = 1:N
            
        rt1(m,n,1)=rt1(m,n,1)+piga(k).*((1/2).*(gamma(n)./rt0(m,n,1)).*pila*((2.*lambda.*lambda(m))./caplambda.*(((gamma(n)./rt0(m,n,1))+(gamma(k)./rt0(:,k,1))).^(-1))));
        
        end
    
      end
      
    end
    
    %replace pre-iteration values with their post-iteration counterparts
    test_rt              = norm(rt1-rt0)/norm(rt0);
    rt0                  = rt1;
    iter_rt              = iter_rt + 1 
    
end

% finalize the iteration
rt                       = rt1; % the final matrix of r-tilde

% calculate inventory aversion based on formula on p. 50
IA                       = zeros(M,N,1); %matrix of inventory aversion
for n=1:N
    
    for m=1:M
        
        IA(m,n,1)        = r.*(sigmaD^2).*gamma(n)./rt(m,n,1);
        
    end
    
end


%__________________________________________________________________________
%
% Generate Figure 1
%__________________________________________________________________________

CommonLineWidth           = 1.5;
tickfont                  = ['tiny'];
labelfont                 = ['footnotesize'];


figure(1)
smps_t=transpose(IA);
clims = [ min(min(IA)) max(max(IA)) ];
imagesc(lambda,gamma,smps_t,clims)
set(gca,'ydir','normal')
colorbar()
xlabel('\lambda');
ylabel('\gamma');
title('inventory aversion');
